#------------------- Boulder::Cursor---------------

# A simple iterator on a Stone.
package Stone::Cursor;

# New expects a Stone object as its single
# parameter.
sub new {
    my($package,$stone) = @_;
    die "Boulder::Cursor: expect a Stone object parameter"
	unless ref($stone);

    my $self = bless {'stone'=>$stone},$package;
    $self->reset;
    $stone->_register_cursor($self,'true');
    return $self;
}

# This procedure does a breadth-first search
# over the entire structure.  It returns an array that looks like this
# (key1[index1].key2[index2].key3[index3],value)
sub each {
    my $self = shift;
    my $stack = $self->{'stack'};

    my($found,$key,$value);
    my $top = $stack->[$#{$stack}];
    while ($top && !$found) {
	$found++ if ($key,$value) = $top->next;
	if (!$found) {		# this iterator is done
	    pop @{$stack};
	    $top = $stack->[$#{$stack}];
	    next;
	}
	if (ref($value)=~/Stone/) { # found another record to begin iterating on
	    if (%{$value}) {
		undef $found;
		$top = $value->cursor;
		push @{$stack},$top;
		next;
	    } else {
		undef $value;
	    }
	}
    }
    unless ($found) {
	$self->reset;
	return ();
    }
    my @keylist = map {($_->{'keys'}->[$_->{'hashindex'}]) 
			   . "[" . ($_->{'arrayindex'}-1) ."]"; } @{$stack};
    return (join(".",@keylist),$value);
}

sub reset {
    my $self = shift;
    $self->{'arrayindex'} = 0;
    $self->{'hashindex'} = 0;
    $self->{'keys'}=[$self->{'stone'}->tags];
    $self->{'stack'}=[$self];
}

sub DESTROY {
    my $self = shift;
    if (ref $self->{'stone'}) {
	$self->{'stone'}->_register_cursor($self,undef);
    }
}

# Next will return the next index in its Stone object,
# indexing first through the members of the array, and then through
# the individual keys.  When iteration is finished, it resets itself
# and returns an empty array.
sub next {
    my $self = shift;
    my($arrayi,$hashi,$stone,$keys) = ($self->{'arrayindex'},
				       $self->{'hashindex'},
				       $self->{'stone'},
				       $self->{'keys'});
    unless ($stone->exists($keys->[$hashi],$arrayi)) {
	$self->{hashindex}=++$hashi;
	$self->{arrayindex}=$arrayi=0;
	unless (defined($keys->[$hashi]) &&
		defined($stone->get($keys->[$hashi],$arrayi))) {
	    $self->reset;
	    return ();
	}
    }
    $self->{arrayindex}++;
    return ($keys->[$hashi],$stone->get($keys->[$hashi],$arrayi));
}


1;
